/* 	$Id$	 */

/* Copyright (c) 2002,2003,2004 Arnaud Legrand. All rights reserved.        */

/* This program is free software; you can redistribute it and/or modify it
 * under the terms of the license (GNU LGPL) which comes with this package. */

#ifndef MSG_DATATYPE_H
#define MSG_DATATYPE_H
#include "xbt/misc.h"

SG_BEGIN_DECL()

/* ******************************** Host ************************************ */
/** @defgroup m_datatypes_management_details Details on MSG datatypes
    @ingroup  m_datatypes_management*/

typedef struct simdata_host *simdata_host_t;
/** @brief Host datatype 
    @ingroup m_datatypes_management_details */
typedef struct m_host {
  char *name;			/**< @brief host name if any */
  simdata_host_t simdata;	/**< @brief simulator data */
  void *data;			/**< @brief user data */
} s_m_host_t;
/** @brief Host datatype  
    @ingroup m_datatypes_management

    A <em>location</em> (or <em>host</em>) is any possible place where
    a process may run. Thus it is represented as a <em>physical
    resource with computing capabilities</em>, some <em>mailboxes</em>
    to enable running process to communicate with remote ones, and
    some <em>private data</em> that can be only accessed by local
    process.

    \see m_host_management
  @{ */
typedef struct m_host *m_host_t;
/** @} */
/* ******************************** Task ************************************ */

typedef struct simdata_task *simdata_task_t;
/** @brief Task datatype 
    @ingroup m_datatypes_management_details */
typedef struct m_task {
  char *name;			/**< @brief task name if any */
  simdata_task_t simdata;	/**< @brief simulator data */
  void *data;			/**< @brief user data */
} s_m_task_t;
/** @brief Task datatype  
    @ingroup m_datatypes_management 

    A <em>task</em> may then be defined by a <em>computing
    amount</em>, a <em>message size</em> and some <em>private
    data</em>.
    \see m_task_management
  @{ */
typedef struct m_task  *m_task_t;

/** \brief Default value for an uninitialized #m_task_t.
    \ingroup m_datatypes_management 
*/
#define MSG_TASK_UNINITIALIZED NULL

/** @} */

/* ****************************** Process *********************************** */
typedef struct simdata_process *simdata_process_t;
/** @brief Process datatype 
    @ingroup m_datatypes_management_details @{ */
typedef struct m_process {
  char *name;			/**< @brief process name if any */
  simdata_process_t simdata;	/**< @brief simulator data */
  void *data;			/**< @brief user data */
} s_m_process_t;
/** @} */
/** @brief Agent datatype  
    @ingroup m_datatypes_management 

    An agent may be defined as a <em>code</em>, with some <em>private
    data</em>, executing in a <em>location</em>.
    \see m_process_management
  @{ */
typedef struct m_process *m_process_t;
/** @} */

/** @brief Agent code
    @ingroup m_datatypes_management 
    The code of an agent is a m_process_code_t, i.e. a function with no arguments 
    returning no value.
    \see m_process_management
  @{ */
typedef int(*m_process_code_t)(int argc,char *argv[]) ;
/** @} */

/* ********************************* Channel ******************************** */
/** @brief Channel datatype  
    @ingroup m_datatypes_management 

    A <em>channel</em>  is a number and identifies a mailbox type (just as a 
    port number does).
    \see m_channel_management
   @{ */
typedef int m_channel_t;
/** @} */

/* ***************************** Error handling ***************************** */
/** @brief Error handling 
    @ingroup m_datatypes_management 
    @{
*/
typedef enum {
  MSG_OK = 0,  /**< @brief Everything is right. Keep on going this way ! */
  MSG_WARNING, /**< @brief Mmmh! Something must be not perfectly clean. But I
      may be a paranoid freak... ! */
  MSG_TRANSFER_FAILURE, /**< @brief There has been a problem during you task
      transfer. Either the network is down or the remote host has been
      shutdown. */
  MSG_HOST_FAILURE, /**< @brief System shutdown. The host on which you are
      running has just been rebooted. Free your datastructures and
      return now !*/
  MSG_TASK_CANCELLED, /**< @brief Cancelled task. This task has been cancelled 
			by somebody!*/
  MSG_FATAL /**< @brief You've done something wrong. You'd better look at it... */
} MSG_error_t;
/** @} */

SG_END_DECL()
#endif
